package se.su.dsv.scipro.plagiarism.urkund;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.workerthreads.AbstractWorker;

import javax.inject.Inject;
import javax.inject.Provider;
import java.time.Instant;

public class StatusPollingWorker extends AbstractWorker {
    private static final Logger LOG = LoggerFactory.getLogger(StatusPollingWorker.class);

    private final UrkundSubmissionRepository urkundSubmissionRepository;
    private final UrkundApi urkundApi;
    private final Provider<UrkundSettings> urkundSettingsProvider;

    @Inject
    StatusPollingWorker(
            final UrkundSubmissionRepository urkundSubmissionRepository,
            final UrkundApi urkundApi,
            final Provider<UrkundSettings> urkundSettingsProvider)
    {
        this.urkundSubmissionRepository = urkundSubmissionRepository;
        this.urkundApi = urkundApi;
        this.urkundSettingsProvider = urkundSettingsProvider;
    }

    @Override
    protected void doWork() {
        if (!urkundSettingsProvider.get().isEnabled()) {
            return;
        }
        for (UrkundSubmission submission : urkundSubmissionRepository.findNotDone()) {
            if (submission.getNextPoll().isBefore(Instant.now())) {
                final UrkundResponse urkundResponse = urkundApi.submissionInfo(
                        submission.getReceiver().getEmailAddress(),
                        submission.getFileDescription());
                final boolean updated = urkundResponse.fold(
                        errorInfo -> false,
                        submissionInfo -> {
                            final UrkundSubmission.State state = Util.getState(submissionInfo.getStatus().getState());
                            if (state != submission.getState()) {
                                submission.setState(state);
                                submission.setMessage(submissionInfo.getStatus().getUserFriendlyMessage());
                                submissionInfo.getReport().map(ReportInfo::getReportUrl).ifPresent(submission::setReportUrl);
                                return true;
                            }
                            return false;
                        });
                final PollingDelay next = submission.getPollingDelay().next();
                final Instant nextPoll = submission.getNextPoll().plus(next.getDuration());
                submission.setPollingDelay(next);
                submission.setNextPoll(nextPoll);
                if (updated) {
                    if (submission.getState().isFinal()) {
                        LOG.info("{} reached the final state {}", submission, submission.getState());
                    }
                    else {
                        LOG.info("{} reached {} state, will poll again in {}", submission, submission.getState(), submission.getPollingDelay());
                    }
                }
                urkundSubmissionRepository.save(submission);
            }
        }
        //get all submission who has not reached a final status
        //check submission time + delay (do i need timestamp for last check? probably)
        //if time > now; skip
        //poll status
        //if new status update submission
        //if not final status increase poll delay
    }
}
